@@ -1,5 +1,5 @@
 mod line_segment;
-mod point;
+pub(crate) mod point;
 pub mod polygon;
 mod triangle;
 mod vector;
@@ -10,11 +10,12 @@ pub use triangle::*;
 // pub use crate::polygon::EdgeIter as testing;
 
 #[doc(inline)]
-pub use crate::data::polygon::{ConvexPolygon, Polygon};
+pub use crate::data::polygon::{Polygon, PolygonConvex};
 pub use crate::transformation::Transform;
 pub use point::Point;
 pub use vector::{Vector, VectorView};
 
+#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord)]
 pub enum PointLocation {
   Inside,
   OnBoundary,
