@@ -1,6 +1,9 @@
 // use claim::debug_assert_ok;
 use num_rational::BigRational;
+use num_traits::*;
+use ordered_float::{FloatIsNan, NotNan, OrderedFloat};
 use std::borrow::Borrow;
+use std::iter::Sum;
 use std::ops::*;
 
 use crate::array::Orientation;
@@ -24,19 +27,26 @@ pub struct Polygon<T, P = ()> {
 }
 
 impl<T> Polygon<T> {
-  pub fn new(points: Vec<Point<T, 2>>) -> Result<Polygon<T>, Error>
+  pub fn new_unchecked(points: Vec<Point<T, 2>>) -> Polygon<T>
   where
     T: PolygonScalar,
   {
     let len = points.len();
     let mut meta = Vec::with_capacity(len);
     meta.resize(len, ());
-    let p = Polygon {
+    Polygon {
       points,
       boundary: len,
       holes: vec![],
       meta,
-    };
+    }
+  }
+
+  pub fn new(points: Vec<Point<T, 2>>) -> Result<Polygon<T>, Error>
+  where
+    T: PolygonScalar,
+  {
+    let p = Self::new_unchecked(points);
     p.validate()?;
     Ok(p)
   }
@@ -168,7 +178,8 @@ impl<T, P> Polygon<T, P> {
 
   pub fn cast<U, F>(self, f: F) -> Polygon<U, P>
   where
-    T: PolygonScalar,
+    T: Clone,
+    U: Clone,
     F: Fn(T) -> U + Clone,
   {
     let pts = self.points.into_iter().map(|p| p.cast(f.clone())).collect();
