@@ -19,12 +19,12 @@ use crate::{Error, PolygonScalar};
 use super::Polygon;
 
 #[derive(Debug, Clone)]
-pub struct ConvexPolygon<T, P = ()>(Polygon<T, P>);
+pub struct PolygonConvex<T, P = ()>(Polygon<T, P>);
 
 ///////////////////////////////////////////////////////////////////////////////
-// ConvexPolygon
+// PolygonConvex
 
-impl<T, P> ConvexPolygon<T, P>
+impl<T, P> PolygonConvex<T, P>
 where
   T: PolygonScalar,
 {
@@ -33,15 +33,17 @@ where
   /// # Safety
   /// The input polygon has to be strictly convex, ie. no vertices are allowed to
   /// be concave or colinear.
-  pub unsafe fn new_unchecked(poly: Polygon<T, P>) -> ConvexPolygon<T, P> {
-    let convex = ConvexPolygon(poly);
+  pub fn new_unchecked(poly: Polygon<T, P>) -> PolygonConvex<T, P> {
+    let convex = PolygonConvex(poly);
     debug_assert_ok!(convex.validate());
     convex
   }
 
-  // O(log n)
+  /// $O(log n)$
+  ///
+  /// <iframe src="https://web.rgeometry.org:20443/loader.html?hash=mV9xetnc_IU="></iframe>
   pub fn locate(&self, pt: &Point<T, 2>) -> PointLocation {
-    debug_assert_ok!(self.validate());
+    // debug_assert_ok!(self.validate());
     let poly = &self.0;
     let p0 = poly.vertex(0);
     let mut lower = 1;
@@ -76,27 +78,25 @@ where
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-// ConvexPolygon<BigRational>
+// PolygonConvex<BigRational>
 
-impl ConvexPolygon<BigRational> {
+impl PolygonConvex<BigRational> {
   /// ```no_run
   /// # use rgeometry_wasm::playground::*;
   /// # use rgeometry::data::*;
   /// # let convex = {
-  /// let mut rng = rand::thread_rng();
-  /// ConvexPolygon::random(3, 1000, &mut rng)
+  /// PolygonConvex::random(3, 1000, &mut rand::thread_rng())
   /// # };
-  /// # render_polygon(convex.into());
-  /// # return ()
+  /// # render_polygon(&convex);
   /// ```
-  /// <iframe src="https://web.rgeometry.org:20443/loader.html?hash=36XCQBE0Yok="></iframe>
-  pub fn random<R>(n: usize, max: usize, rng: &mut R) -> ConvexPolygon<BigRational>
+  /// <iframe src="https://web.rgeometry.org:20443/loader.html?gist=037a23f8391390df8560a2043a14121e"></iframe>
+  pub fn random<R>(n: usize, max: usize, rng: &mut R) -> PolygonConvex<BigRational>
   where
     R: Rng + ?Sized,
   {
     if n < 3 {
       // Return Result<P, Error> instead?
-      return ConvexPolygon::random(3, max, rng);
+      return PolygonConvex::random(3, max, rng);
     }
     let vs = {
       let mut vs = random_vectors(n, max, rng);
@@ -112,7 +112,7 @@ impl ConvexPolygon<BigRational> {
       .collect();
     let n_vertices = (*vertices).len();
     debug_assert_eq!(n_vertices, n);
-    let p = Polygon::new(vertices).unwrap();
+    let p = Polygon::new_unchecked(vertices);
     for i in 0..n {
       if p.vertex_orientation(i as isize) != Orientation::CounterClockWise {
         return Self::random(n, max, rng);
@@ -124,65 +124,84 @@ impl ConvexPolygon<BigRational> {
       One::one(),
       BigInt::from_usize(max).unwrap(),
     ));
-    ConvexPolygon(s * t * p)
+    PolygonConvex::new_unchecked(s * t * p)
   }
 }
 
 ///////////////////////////////////////////////////////////////////////////////
 // Trait Implementations
 
-impl<T: PolygonScalar, P> Deref for ConvexPolygon<T, P> {
-  type Target = Polygon<T,P>;
+impl<T: PolygonScalar, P> Deref for PolygonConvex<T, P> {
+  type Target = Polygon<T, P>;
   fn deref(&self) -> &Self::Target {
     self.polygon()
   }
 }
 
-impl<T, P> From<ConvexPolygon<T, P>> for Polygon<T, P> {
-  fn from(convex: ConvexPolygon<T, P>) -> Polygon<T, P> {
+impl<T, P> From<PolygonConvex<T, P>> for Polygon<T, P> {
+  fn from(convex: PolygonConvex<T, P>) -> Polygon<T, P> {
     convex.0
   }
 }
 
-impl<'a, T, P> From<&'a ConvexPolygon<T, P>> for &'a Polygon<T, P> {
-  fn from(convex: &'a ConvexPolygon<T, P>) -> &'a Polygon<T, P> {
+impl<'a, T, P> From<&'a PolygonConvex<T, P>> for &'a Polygon<T, P> {
+  fn from(convex: &'a PolygonConvex<T, P>) -> &'a Polygon<T, P> {
     &convex.0
   }
 }
 
-impl Distribution<ConvexPolygon<BigRational>> for Standard {
-  fn sample<R: Rng + ?Sized>(&self, rng: &mut R) -> ConvexPolygon<BigRational> {
-    ConvexPolygon::random(100, usize::MAX, rng)
+impl Distribution<PolygonConvex<BigRational>> for Standard {
+  fn sample<R: Rng + ?Sized>(&self, rng: &mut R) -> PolygonConvex<BigRational> {
+    PolygonConvex::random(100, usize::MAX, rng)
   }
 }
 
 ///////////////////////////////////////////////////////////////////////////////
 // Helper functions
 
-// Property: random_between(n, max, &mut rng).iter().sum::<usize>() == max
-fn random_between<R>(n: usize, max: usize, rng: &mut R) -> Vec<usize>
+// Property: random_between(n, max, &mut rng).sum::<usize>() == max
+fn random_between_iter<R>(n: usize, max: usize, rng: &mut R) -> impl Iterator<Item = usize>
 where
   R: Rng + ?Sized,
 {
-  debug_assert!(n > 0);
+  assert!(n > 0);
   assert!(n <= max);
-  if max == n {
-    return vec![1; n];
-  }
   let mut pts = BTreeSet::new();
   while pts.len() < n - 1 {
     pts.insert(rng.gen_range(1..max));
   }
-  let mut from = 0;
-  let mut out = Vec::new();
-  for x in pts.iter() {
-    out.push(x - from);
-    from = *x;
-  }
-  out.push(max - from);
-  out
+  pts.insert(max);
+  pts.into_iter().scan(0, |from, x| {
+    let out = x - *from;
+    *from = x;
+    Some(out)
+  })
 }
 
+// Property: random_between(n, max, &mut rng).iter().sum::<usize>() == max
+// fn random_between<R>(n: usize, max: usize, rng: &mut R) -> Vec<usize>
+// where
+//   R: Rng + ?Sized,
+// {
+//   assert!(n > 0);
+//   assert!(n <= max);
+//   if max == n {
+//     return vec![1; n];
+//   }
+//   let mut pts = BTreeSet::new();
+//   while pts.len() < n - 1 {
+//     pts.insert(rng.gen_range(1..max));
+//   }
+//   pts.insert(max);
+//   let mut from = 0;
+//   let mut out = Vec::new();
+//   for x in pts.iter() {
+//     out.push(x - from);
+//     from = *x;
+//   }
+//   out
+// }
+
 // Property: random_between_zero(10, 100, &mut rng).iter().sum::<isize>() == 0
 // Property: random_between_zero(10, 100, &mut rng).iter().all(|v| !v.is_zero())
 fn random_between_zero<R>(n: usize, max: usize, rng: &mut R) -> Vec<BigInt>
@@ -191,26 +210,15 @@ where
 {
   assert!(n >= 2);
   let n_positive = rng.gen_range(1..n); // [1;n[
-  dbg!(n_positive);
-  let positive = random_between(n_positive, max, rng)
-    .into_iter()
-    .map(BigInt::from)
-    .collect();
   let n_negative = n - n_positive;
-  let negative: Vec<BigInt> = random_between(n_negative, max, rng)
-    .into_iter()
+  assert!(n_positive + n_negative == n);
+  let positive = random_between_iter(n_positive, max, rng).map(BigInt::from);
+  let negative = random_between_iter(n_negative, max, rng)
     .map(BigInt::from)
-    .map(Neg::neg)
-    .collect();
-  let mut result = [positive, negative].concat();
+    .map(Neg::neg);
+  let mut result: Vec<BigInt> = positive.chain(negative).collect();
   result.shuffle(rng);
   result
-  // random_between(n, max, rng)
-  //   .into_iter()
-  //   .map(BigInt::from)
-  //   .zip(random_between(n, max, rng).into_iter().map(BigInt::from))
-  //   .map(|(a, b)| a - b)
-  //   .collect()
 }
 
 // Random vectors that sum to zero.
@@ -240,8 +248,8 @@ mod tests {
 
   use ordered_float::NotNan;
 
-  impl Arbitrary for ConvexPolygon<BigRational> {
-    type Strategy = Just<ConvexPolygon<BigRational>>;
+  impl Arbitrary for PolygonConvex<BigRational> {
+    type Strategy = Just<PolygonConvex<BigRational>>;
     type Parameters = ();
     fn arbitrary_with(_params: ()) -> Self::Strategy {
       Self::arbitrary()
@@ -250,14 +258,14 @@ mod tests {
       let mut rng = rand::thread_rng();
       let n = rng.gen_range(3..=100);
       let max = rng.gen_range(n..=1_000_000_000);
-      let p = ConvexPolygon::random(n, max, &mut rng);
+      let p = PolygonConvex::random(n, max, &mut rng);
       Just(p)
     }
   }
 
   proptest! {
     #[test]
-    fn all_random_convex_polygons_are_valid(poly: ConvexPolygon<BigRational>) {
+    fn all_random_convex_polygons_are_valid(poly: PolygonConvex<BigRational>) {
       prop_assert_eq!(poly.validate(), Ok(()))
     }
 
@@ -265,8 +273,8 @@ mod tests {
     fn sum_to_max(n in 1..1000, max in 0..1_000_000) {
       let mut rng = rand::thread_rng();
       let max = std::cmp::max(max, n);
-      let vecs = random_between(n as usize, max as usize, &mut rng);
-      prop_assert_eq!(vecs.iter().sum::<usize>(), max as usize)
+      let vecs = random_between_iter(n as usize, max as usize, &mut rng);
+      prop_assert_eq!(vecs.sum::<usize>(), max as usize)
     }
 
     #[test]
